import re

LINK_REGEXP = re.compile('<a [^>]*href="([^"]*)"')


def query_results(query: str, page=1):
    from pytorrentsearch.utils import get_url_content, status, min_wait
    from urllib.parse import quote

    page_links: set[str] = set()
    min_waiter = min_wait(5)
    while True:
        for link in page_links:
            yield link
        next(min_waiter)
        status("Fetching Yandex result page...")
        search_url = f"https://yandex.com/search?text={quote(query)}&p={page}"
        content = get_url_content(search_url)
        page_links = set()
        for link in LINK_REGEXP.findall(content):
            # if link.startswith("http"):
            page_links.add(link)
        if len(page_links) == 0:
            break
        if "smartcaptcha" in " ".join(page_links):
            status("Yandex asked captcha, giving up")
            break



"""CLI interface for pytorrentsearch project.

Be creative! do whatever you want!

- Install click or typer and create a CLI app
- Use builtin argparse
- Start a web application
- Import things from your .base module
"""


def main():  # pragma: no cover
    from argparse import ArgumentParser
    from pytorrentsearch.search import yandex, google, duckduckgo
    from pytorrentsearch.utils import multi_iterator_pooler
    from pytorrentsearch.miner import mine_magnet_links
    from pytorrentsearch.miner import prettyprint_magnet

    parser = ArgumentParser("pytorrentsearch")
    parser.add_argument("query", type=str)
    args = parser.parse_args()
    print(args)

    if args.query.startswith("http"):
        for magnet in mine_magnet_links(args.query):
            prettyprint_magnet(magnet)
        exit(0)

    iterators = []
    for search_backend in [yandex, google, duckduckgo]:
        iterators.append(search_backend.query_results(args.query))

    visited_urls = {}

    url_iterator = multi_iterator_pooler(*iterators)
    for url in url_iterator:
        if visited_urls.get(url) is not None:
            continue
        try:
            magnets = mine_magnet_links(url)
            for magnet in magnets:
                prettyprint_magnet(magnet)
        except Exception as e:
            print(e)
            continue
        finally:
            visited_urls[url] = True



def status(message: str):
    from sys import stderr

    print(f"[*] {message}", file=stderr)


def request(url: str, timeout=10):
    from urllib.request import urlopen, Request

    req = Request(
        url,
        headers={
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36"  # noqa: E501
        },
    )
    res = urlopen(req, timeout=timeout)
    return res


def get_url_content(url: str, timeout=10):
    return request(url, timeout=timeout).read().decode("utf8")


def multi_iterator_pooler(*iterators):
    import queue
    from threading import Thread
    from time import sleep

    q = queue.Queue(maxsize=len(iterators))
    threads = []

    def worker(iterator):
        while True:
            q.put(next(iterator))

    for iterator in iterators:
        threads.append(Thread(target=worker, args=[iterator]))
    for thread in threads:
        thread.start()
    while True:
        if not q.empty():
            obj = q.get()
            q.task_done()
            yield obj
        keep_going = False
        for thread in threads:
            if thread.is_alive():
                keep_going = True
        if keep_going:
            sleep(0.1)


def min_wait(seconds):
    from time import time, sleep

    last_time = time()
    yield None
    while True:
        timediff = time() - last_time
        if timediff < seconds:
            sleep(seconds - timediff)
        yield None



import re

MAGNET_REGEXP = re.compile("magnet:\\?xt=[^\"']*")

nontorrent_blockwords = [
    "lumendatabase.org",
    "disney.com.br",
    "reddit.com",
    "facebook.com",
    "google.com",
    "youtube.com",
    "youtu.be",
    "wikipedia.org",
    "instagram.com",
    "ifunny.co",
    "9gag.com",
]


def is_common_nontorrent_site(url: str):
    for nontorrent_blockword in nontorrent_blockwords:
        if url.find(nontorrent_blockword) > 0:
            return True
    return False


def mine_magnet_links(url: str):
    from pytorrentsearch.utils import get_url_content, status
    from urllib.parse import unquote

    if is_common_nontorrent_site(url):
        status(f"[crawler/ENONTORRNET] {url}")
        return []
    status(f"[crawler/fetch] {url}")
    content = get_url_content(url)
    ret = []
    for occurence in MAGNET_REGEXP.findall(content):
        link = unquote(occurence)
        link = link.replace("&#038;", "&")
        link = link.replace("&amp;", "&")
        ret.append(link)
    return ret


def parse_magnet_link(url: str):
    from urllib.parse import urlparse, parse_qs

    query = urlparse(url).query
    query_params = parse_qs(query)
    info_hash = query_params["xt"][0].replace("urn:", "").replace("btih:", "")
    name = "< NO NAME >"
    if query_params.get("dn") is not None:
        name = query_params["dn"][0]
    trackers = []
    if query_params.get("tr") is not None:
        trackers = query_params["tr"]
    return dict(info_hash=info_hash, name=name, trackers=trackers)


def prettyprint_magnet(magnet: str):
    parsed = parse_magnet_link(magnet)
    len_trackers = len(parsed["trackers"])
    print(
        f"{parsed['name']}\nTrackers: {str(len_trackers).rjust(3)} InfoHash: {parsed['info_hash']}\n{magnet}\n"  # noqa: E501
    )



import re

LINK_REGEXP = re.compile('<a [^>]*href="([^"]*)"')


def query_results(query: str, page=1):
    from urllib.parse import quote
    from pytorrentsearch.utils import get_url_content, status, min_wait

    page_links: set[str] = set()
    min_waiter = min_wait(5)
    while True:
        for link in page_links:
            yield link
        next(min_waiter)
        status("Fetching Google result page...")
        search_url = f"https://www.google.com/search?q={quote(query)}&start={(page-1)*20}"  # noqa: E501
        content = get_url_content(search_url)
        page_links = set()
        for link in LINK_REGEXP.findall(content):
            if link.startswith("/url"):
                if link.find("http") != -1:
                    link = link[link.find("http") :]
                else:
                    continue
                if link.find("&amp"):
                    link = link[0 : link.find("&amp")]
            if link.startswith("http") and link.find("google.com") == -1:
                page_links.add(link)
        if len(page_links) == 0:
            break
